// [y,b] = smodtm(t,x,tt,nue,bin)
//
// use kernel-smooth method for curve
//
// input
//
// t: original curve time vector, n*1 or 1*n
// x: original curve data, m curves, m*n
// tt: new time vector, nav*1 or 1*nav
// nue: [optional] order of derivative (0-2), output y is f, f', or f''. 
//      default value: nue = 0 (output the smoothed curve data, f)
//
// bin: [optional] user-input global bandwidth
//
//
// output
//
// y: m*nav, smoothed curve data
// b: [optinal] optimal bindwidth for m curves, m*1
//
//
// Jinwei Gu. 2005/12/6
// jwgu@cs.columbia.edu

#include "mex.h" 
#include "math.h"
#include "stdio.h"
#include "f2c.h"

#pragma comment(lib, "vcf2c.lib")
#pragma comment(linker, "/nodefaultlib:msvcrt")

void mexFunction(int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[]) 
{ 
    double *t, *dat, *tt, *x, *y, *s, *wn, *binv, *bout, *yout;
    double w1[1200], tu, to, bop, bin, sig;
    int i, j, n, m, n1, n2, nav, m1=400; 
    int nue, kord, ihom, irnd, ismo;
    
    extern /* Subroutine */ int glkern_(doublereal *t, doublereal *x, integer *n, 
	doublereal *tt, integer *m, integer *ihom, integer *nue, integer *
	kord, integer *irnd, integer *ismo, integer *m1, doublereal *tu, 
	doublereal *to, doublereal *s, doublereal *sig, doublereal *wn, 
	doublereal *w1, doublereal *b, doublereal *y);
    
    if(nrhs<3 || nrhs>5)
        mexErrMsgTxt("must have 3 inputs");
    
    if(nlhs<1 || nlhs>2)
        mexErrMsgTxt("must have 2 outputs");
    
    // t    # time vector n*1
    // n    # number of points on all curves
    // m    # number of curves
    t = mxGetPr(prhs[0]);
    n1 = mxGetM(prhs[0]);
    n2 = mxGetN(prhs[0]);
        
    // dat   # data m*n matrix
    dat = mxGetPr(prhs[1]);
    m = mxGetM(prhs[1]);
    n = mxGetN(prhs[1]);    
           
    if(__max(n1,n2)!=n || __min(n1,n2)!=1)
        mexErrMsgTxt("invalid input t or dat");
    
    // tt   # new time vector nav*1
    // nav  # number of points on each of the output curve
    tt = mxGetPr(prhs[2]);
    n1 = mxGetM(prhs[2]);
    n2 = mxGetN(prhs[2]);
    nav = __max(n1,n2);
    if(__min(n1,n2)!=1)
        mexErrMsgTxt("invalid input tt");
    
    // nue  # options of output: f, f', or f''
    nue = 0;
    if(nrhs>=4)
    {
        nue = (int)(mxGetScalar(prhs[3]));
        if(nue!=0 && nue!=1 && nue!=2)
            mexErrMsgTxt("invalid nue. make sure nue = {0,1,2}");
    } 
    
    // binv # user-input bandwidth for each of the m curves
    // bin  # user-input global bandwidth    
    binv = NULL; bin = 0;
    if(nrhs==5)
    {
        if(m==1)
        {
            bin = mxGetScalar(prhs[4]);
        }
        else
        {
            binv = mxGetPr(prhs[4]);
            n1 = mxGetM(prhs[4]);
            n2 = mxGetN(prhs[4]);
            if(__min(n1,n2)!=1 || __max(n1,n2)!=m)
                mexErrMsgTxt("invalid input bin (should be m*1 vector, m is curve #");
        }
    }
    
    // setting up the working arrays
    wn = mxCalloc((1+n)*5, sizeof(double));
    s = mxCalloc((1+n), sizeof(double));
    y = mxCalloc(nav, sizeof(double));
    x = mxCalloc(n, sizeof(double));
    
    // output
    plhs[0] = mxCreateDoubleMatrix(m,nav,mxREAL);    
    yout = mxGetPr(plhs[0]);
    
    if(nlhs==2)
    {
        plhs[1] = mxCreateDoubleMatrix(m,1,mxREAL);
        bout = mxGetPr(plhs[1]);
    }
    
    // main
    for(i=0; i<m; i++)
    {
        ihom = 0;    
        kord = nue + 2;
        irnd = 0;
        
        tu = t[0] + 0.1*(t[n-1]-t[0]);
        to = t[0] + 0.9*(t[n-1]-t[0]);
                   
        s[0] = 1.0f;
        s[n] = 0.0f;
        sig = 0.0f;        
        ismo = 0;                   
        bop = 0.0f;
        
        if(nrhs==5)
        {
            ismo = 1;
            
            if(m==1)
                bop = bin;
            else
                bop = binv[i];
        }
                 
        for(j=0; j<nav; j++)
        {
            y[j] = 0.0f;  
        }
        for(j=0; j<n; j++)
        {
            x[j] = dat[j*m+i];
        }
                
        glkern_(t,x,&n,tt,&nav,&ihom,&nue,&kord,&irnd,&ismo,&m1,
                &tu,&to,s,&sig,wn,w1,&bop,y);
        
        for(j=0; j<nav; j++)
        {
            yout[j*m+i] = y[j];
        }
        
        if(nlhs==2) bout[i] = bop;
    }
}
